#import "ASAlarm.h"
#import "ASAppDelegate.h"

@implementation ASAlarm

- (id)initWithActivationDate:(NSDate *)activationDate
{
    self = [super init];
    if (self) {
        self.activationDate = activationDate;
    }
    return self;
}

- (id)init
{
    //
    // Default to next minute with seconds set to zero
    //
    NSDate *now = [NSDate date];
    NSDateComponents *seconds = [[NSCalendar currentCalendar]
                                 components:NSSecondCalendarUnit fromDate:now];
    NSDate *nextMinute = [now dateByAddingTimeInterval:-seconds.second+60.0];
    
    self = [self initWithActivationDate:nextMinute];
    return self;
}

- (void)dealloc
{
    if (self.audioPlayer && [self.audioPlayer isPlaying]) {
        [self.audioPlayer stop];
    }
}

- (void)fire
{
    [NSApp activateIgnoringOtherApps:YES];
    
    //
    // Start playing audio file
    //
    if (self.audioFile) {
        self.audioPlayer = [[AVAudioPlayer alloc]
                            initWithContentsOfURL:self.audioFile error:nil];
        if (self.audioPlayer) {
            [self.audioPlayer play];
        }
    }
    
    //
    // Show alert message
    //
    NSAlert *alarmDialog = [[NSAlert alloc] init];
    alarmDialog.messageText = ([self.message length] > 0) ? self.message : @"Alarm";
    [alarmDialog runModal];
}

- (void)timerFired:(NSTimer *)aTimer
{
    [self deactivate];
    [self fire];
    
    ASAppDelegate *appDelegate = (ASAppDelegate *) [NSApp delegate];
    [appDelegate unregisterAlarm:self];
}

- (void)activate
{
    [self receiveWillSleepEvents];
    [self installTimer];
}

- (void)deactivate
{
    [self dontReceiveWillSleepEvents];
    [self dontReceiveDidWakeEvents];
    [self uninstallTimer];
}

/*
 * Automatically called when the system will go down to sleep.
 */
- (void)workspaceWillSleep:(NSNotification *)notification
{
    [self dontReceiveWillSleepEvents];
    [self receiveDidWakeEvents];
    [self uninstallTimer];
}

/*
 * Automatically called when the system wakes up from sleep.
 */
- (void)workspaceDidWake:(NSNotification *)notification
{
    [self dontReceiveDidWakeEvents];
    [self receiveWillSleepEvents];
    [self installTimer];
}

- (void)receiveWillSleepEvents
{
    NSNotificationCenter *nc = [[NSWorkspace sharedWorkspace]
                                notificationCenter];
    [nc addObserver:self
           selector:@selector(workspaceWillSleep:)
               name:NSWorkspaceWillSleepNotification
             object:NULL];
}

- (void)dontReceiveWillSleepEvents
{
    NSNotificationCenter *nc = [[NSWorkspace sharedWorkspace]
                                notificationCenter];
    [nc removeObserver:self
                  name:NSWorkspaceWillSleepNotification
                object:NULL];
}

- (void)receiveDidWakeEvents
{
    NSNotificationCenter *nc = [[NSWorkspace sharedWorkspace]
                                notificationCenter];
    [nc addObserver:self
           selector:@selector(workspaceDidWake:)
               name:NSWorkspaceDidWakeNotification
             object:NULL];
}

- (void)dontReceiveDidWakeEvents
{
    NSNotificationCenter *nc = [[NSWorkspace sharedWorkspace]
                                notificationCenter];
    [nc removeObserver:self
                  name:NSWorkspaceDidWakeNotification
                object:NULL];
}

- (void)installTimer
{
    NSTimeInterval interval = [self.activationDate timeIntervalSinceNow];
    if (interval < 0.0)
        interval = 0.0;

    self.timer = [NSTimer scheduledTimerWithTimeInterval:interval
                                                  target:self
                                                selector:@selector(timerFired:)
                                                userInfo:nil
                                                 repeats:NO];
}

- (void)uninstallTimer
{
    [self.timer invalidate];
    self.timer = nil;
}

@end
