#import "ASAlarmWindowController.h"
#import "ASAppDelegate.h"
#import "ASAlarm.h"

@implementation ASAlarmWindowController

- (id)initWithAlarm:(ASAlarm *)alarm
{
    self = [super initWithWindowNibName:@"ASAlarmWindow"];
    if (self) {
        alarm.windowController = self;
        self.alarm = alarm;
        [self reset];
    }
    
    return self;
}

- (IBAction)resetFilePath:(id)sender
{
    self.selectedURL = nil;
}

- (IBAction)activate:(id)sender
{
    ASAppDelegate *appDelegate = (ASAppDelegate *) [NSApp delegate];
    [appDelegate registerAlarm:self.alarm];
    [self.alarm setActivationDate:self.activationDate];
    [self.alarm setAudioFile:self.selectedURL];
    [self.alarm setMessage:self.message];
    [self.alarm deactivate];
    [self.alarm activate];
    [self close];
}

/*
 * User selected a path component in the popup menu.
 */
- (IBAction)browse:(id)sender
{
    NSPathComponentCell *clickedCell = nil;
    NSURL *clickedURL = nil;
    
    clickedCell = [self.filePathControl clickedPathComponentCell];
    clickedURL = [clickedCell URL];

    //
    // Let user select another file.
    // TODO: Invoke the 'Choose' menu item instead of using a
    //       custom NSOpenPanel.
    //
    if ([clickedURL isEqual:self.selectedURL] == NO) {
        NSOpenPanel *openPanel = [NSOpenPanel openPanel];
        [openPanel setDirectoryURL:clickedURL];
        [openPanel beginWithCompletionHandler:^(NSInteger result) {
            if (result == NSOKButton && [[openPanel URLs] count] > 0) {
                [self.alarm setAudioFile:[[openPanel URLs] objectAtIndex:0]];
                [self reset];
            }
        }];
    }
}

- (IBAction)changeTime:(id)sender
{
    NSDate *now = [NSDate date];
    if ([now compare:self.activationDate] == NSOrderedDescending) {
        [self.textualDatePicker setTextColor:[NSColor redColor]];
    } else {
        [self.textualDatePicker setTextColor:[NSColor blackColor]];
    }
}

- (IBAction)cancel:(id)sender
{
    [self reset];
    [self close];
}

- (IBAction)remove:(id)sender
{
    ASAppDelegate *appDelegate = (ASAppDelegate *) [NSApp delegate];
    [appDelegate unregisterAlarm:self.alarm];
    [self.alarm deactivate];
    [self close];
}

/*
 * Reset user interface components.
 */
- (void)reset
{
    self.activationDate = [self.alarm activationDate];
    self.selectedURL = [self.alarm audioFile];
}

- (void)windowWillClose:(NSNotification *)notification
{
    [self reset];
}

@end
