#import "ASAppDelegate.h"
#import "ASAlarm.h"
#import "ASAlarmWindowController.h"
#import "NSDate+Formatting.h"

@implementation ASAppDelegate

- (id)init
{
    self = [super init];
    if (self) {
        self.alarms = [[NSMutableArray alloc] init];
        self.alarmWindowControllers = [[NSMutableArray alloc] init];
    }
    return self;
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    [self updateStatusMenu];
}

/*
 * Update the status menu in the menu bar.
 */
- (void)updateStatusMenu
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        NSStatusBar *statusBar = [NSStatusBar systemStatusBar];
        self.appStatusItem = [statusBar
                              statusItemWithLength:NSVariableStatusItemLength];
        [self.appStatusItem setTitle: NSLocalizedString(@"AS",@"")];
        [self.appStatusItem setHighlightMode:YES];
        
        self.appStatusMenu = [[NSMenu alloc] init];
        [self.appStatusItem setMenu:self.appStatusMenu];
    });
    
    @synchronized(self.appStatusMenu) {
        [self.appStatusMenu removeAllItems];
        
        //
        // Create a menu item for each alarm
        //
        if (self.alarms && [self.alarms count] > 0) {
            for (ASAlarm *theAlarm in self.alarms) {
                NSMenuItem *alarmItem = [[NSMenuItem alloc] init];
                [alarmItem bind:NSTitleBinding
                       toObject:theAlarm
                    withKeyPath:@"activationDate.AS_formattedDescription"
                        options:nil];
                alarmItem.target = self;
                alarmItem.action = @selector(alarmSelected:);
                alarmItem.representedObject = theAlarm;
                [self.appStatusMenu addItem:alarmItem];
            }
            
            [self.appStatusMenu addItem:[NSMenuItem separatorItem]];
        }
        
        //
        // Menu item for creating new alarms
        //
        NSMenuItem *newAlarmItem = [[NSMenuItem alloc] init];
        newAlarmItem.title = NSLocalizedString(@"New Alarm...", nil);
        newAlarmItem.target = self;
        newAlarmItem.action = @selector(newAlarm:);
        [self.appStatusMenu addItem:newAlarmItem];
        
        [self.appStatusMenu addItem:[NSMenuItem separatorItem]];
        
        //
        // Menu item for displaying information about the app
        //
        NSMenuItem *aboutItem = [[NSMenuItem alloc] init];
        aboutItem.title = NSLocalizedString(@"About Antisleep", nil);
        aboutItem.target = self;
        aboutItem.action = @selector(showAbout:);
        [self.appStatusMenu addItem:aboutItem];
                
        [self.appStatusMenu addItem:[NSMenuItem separatorItem]];
        
        //
        // Menu item for quitting the app
        //
        NSMenuItem *quitItem = [[NSMenuItem alloc] init];
        quitItem.title = NSLocalizedString(@"Quit", nil);
        quitItem.target = NSApp;
        quitItem.action = @selector(terminate:);
        [self.appStatusMenu addItem:quitItem];
    }
}

- (void)alarmSelected:(id)sender
{
    ASAlarm *alarm = [sender representedObject];
    [self showAlarm:alarm];
}

- (void)showAlarm:(ASAlarm *)alarm
{
    ASAlarmWindowController *awc = alarm.windowController;
    if (!awc) {
        awc = [[ASAlarmWindowController alloc] initWithAlarm:alarm];
        [self.alarmWindowControllers addObject:awc];
    }

    [NSApp activateIgnoringOtherApps:YES];
    [awc showWindow:self];
}

- (void)newAlarm:(id)sender
{
    ASAlarm *alarm = [[ASAlarm alloc] init];
    [self showAlarm:alarm];
}

- (void)showAbout:(id)sender
{
    [NSApp activateIgnoringOtherApps:YES];
    [NSApp orderFrontStandardAboutPanel:self];
}

- (void)registerAlarm:(ASAlarm *)alarm
{
    if (![self.alarms containsObject:alarm]) {
        [self.alarms addObject:alarm];
        [self updateStatusMenu];
    }
}

- (void)unregisterAlarm:(ASAlarm *)alarm
{
    [self.alarms removeObject:alarm];
    [self updateStatusMenu];

    ASAlarmWindowController *awc = alarm.windowController;
    if (awc) {
        [self.alarmWindowControllers removeObject:awc];
    }
}

@end
